﻿using System.Text;
using System.Text.Json;
using System.Text.Json.Serialization;

var serializationOptions = new JsonSerializerOptions()
{
    Converters =
    {
        new JsonStringEnumConverter()
    },
    NumberHandling = JsonNumberHandling.AllowReadingFromString,
};
string jsonString = File.ReadAllText("transcription.json");

TranscriptionJob transcriptionJob = JsonSerializer.Deserialize<TranscriptionJob>(jsonString, serializationOptions);

Dictionary<double, Line> dialogue = new Dictionary<double, Line>();

StringBuilder sb = new StringBuilder();
bool startOfSentence = true;
double sentenceStartTime = 0;

foreach (var channel in transcriptionJob.Results.ChannelLabels.Channels)
{
    foreach (var item in channel.Items)
    {
        if (startOfSentence)
        {
            sentenceStartTime = item.StartTime;
            startOfSentence = false;
        }

        if (item.Type == AudioType.Punctuation)
        {
            sb.Remove(sb.Length - 1, 1); // remove space before punctuation
            sb.Append($"{item.Alternatives[0].Content} ");
            if (item.Alternatives[0].Content != ",") // anything other than a comma is an end of line
            {
                var line = new Line
                {
                    Content = sb.ToString(),
                    StartTime = sentenceStartTime,
                    ChannelLabel = item.ChannelLabel
                };

                bool resultOfAdd = dialogue.TryAdd(sentenceStartTime, line);
                if (!resultOfAdd)
                {
                    Console.WriteLine($"Two lines with the same start time: {sentenceStartTime}");
                    // change the start time to avoid a duplicate key exception
                    dialogue.TryAdd(sentenceStartTime + 0.001, line);
                }
                startOfSentence = true;
                sb.Clear();
            }
        }
        else
        {
            sb.Append($"{item.Alternatives[0].Content} ");
        }
    }
}

foreach (var keyValuePair in dialogue.OrderBy(x => x.Key))
{
    var timeSpan = TimeSpan.FromSeconds(keyValuePair.Key);
    Console.WriteLine($"{timeSpan.ToString(@"hh\:mm\:ss")}: {keyValuePair.Value.ChannelLabel}: {keyValuePair.Value.Content}");
}